@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.210 2005/10/22 18:34:53 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.216 2006/01/19 11:21:33 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -34,6 +34,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <sched.h>
 /* X11 */
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -452,7 +453,9 @@ bacon_video_widget_finalize (GObject *object)
 	BaconVideoWidget *bvw = (BaconVideoWidget *) object;
 
 	if (bvw->priv->xine != NULL) {
+		xine_plugins_garbage_collector (bvw->priv->xine);
 		xine_exit (bvw->priv->xine);
+		bvw->priv->xine = NULL;
 	}
 	if (bvw->priv->cursor != NULL) {
 		gdk_cursor_unref (bvw->priv->cursor);
@@ -1428,6 +1431,8 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 
 	save_data = NULL;
 
+	sched_yield ();
+
 	/* Steal messages from the async queue, if there's an error,
 	 * to use as the error message rather than the crappy errors from
 	 * xine_open() */
@@ -1553,6 +1558,9 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	xine_config_save (bvw->priv->xine, configfile);
 	g_free (configfile);
 
+	if (bvw->priv->vis != NULL)
+		xine_post_dispose (bvw->priv->xine, bvw->priv->vis);
+
 	/* Kill the drivers */
 	if (bvw->priv->vo_driver != NULL)
 		xine_close_video_driver (bvw->priv->xine,
@@ -1561,10 +1569,8 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 		xine_close_audio_driver (bvw->priv->xine,
 				bvw->priv->ao_driver);
 
-	if (bvw->priv->vis != NULL)
-		xine_post_dispose (bvw->priv->xine, bvw->priv->vis);
-
 	/* stop event thread */
+	xine_plugins_garbage_collector (bvw->priv->xine);
 	xine_exit (bvw->priv->xine);
 	bvw->priv->xine = NULL;
 
@@ -1653,6 +1659,7 @@ bacon_video_widget_new (int width, int height,
 					bvw->priv->ao_driver);
 		}
 		xine_exit (bvw->priv->xine);
+		bvw->priv->xine = NULL;
 
 		/* get rid of all our crappety crap */
 		g_source_remove (bvw->priv->tick_id);
@@ -2009,6 +2016,8 @@ bacon_video_widget_open_with_subtitle (BaconVideoWidget *bvw, const char *mrl,
 		err = xine_open (bvw->priv->stream, mrl);
 	}
 
+	xine_plugins_garbage_collector (bvw->priv->xine);
+
 	if (err == 0)
 	{
 		bacon_video_widget_close (bvw);
@@ -2151,6 +2160,12 @@ bacon_video_widget_play (BaconVideoWidget *bvw, GError **gerror)
 	return TRUE;
 }
 
+gboolean
+bacon_video_widget_can_direct_seek (BaconVideoWidget *bvw)
+{
+	return FALSE;
+}
+
 gboolean bacon_video_widget_seek (BaconVideoWidget *bvw, float position,
 		GError **gerror)
 {
@@ -3307,10 +3322,6 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget *bvw, BaconVideoWidgetM
 		string = xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_AUDIOCODEC);
 		break;
-	case BVW_INFO_CDINDEX:
-		string = xine_get_meta_info (bvw->priv->stream,
-				XINE_META_INFO_CDINDEX_DISCID);
-		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -3453,7 +3464,6 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
 	case BVW_INFO_YEAR:
 	case BVW_INFO_VIDEO_CODEC:
 	case BVW_INFO_AUDIO_CODEC:
-	case BVW_INFO_CDINDEX:
 		bacon_video_widget_get_metadata_string (bvw, type, value);
 		break;
 	case BVW_INFO_DURATION:
